package com.zy.omp.config.mqtt;

import com.zy.omp.common.Constant;
import com.zy.omp.pojo.dto.MqttMsgDto;
import com.zy.omp.model.*;
import com.zy.omp.service.*;
import com.zy.omp.utils.*;
import com.zy.omp.utils.CoordTransformUtil;
import com.zy.omp.utils.GaoDeApiUtil;
import com.zy.omp.websocket.WebSocketServer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * MQTT 消息返回处理类
 *
 * @author yang xiao kun
 * create on 2021/1/19
 */
@Slf4j
@Component
public class MqttCallbackHandler {

    @Resource
    private DeviceService deviceService;
    @Resource
    private SetBaseService setBaseService;
    @Resource
    private SetService setService;
    @Resource
    private LocationService deviceLocationService;
    @Resource
    private WebSocketServer webSocketServer;
    @Resource
    private CallRecordsService callRecordsService;
    @Resource
    private MqttLogService mqttLogService;
    @Resource
    private RabbitMQApi rabbitMQApi;

    /**
     * 处理消息
     *
     * @param topic   主题
     * @param payload 消息内容
     */
    void handle(String topic, String payload) {
        //存储日志-接收
        mqttLogService.saveReceiveLog(topic, payload);
        //消息返回JSON转Object
        MqttMsgDto msgDto = MqttMsgDto.parse(payload);
        switch (msgDto.getM()) {
            //设备端注册报文
            case Constant.M_CODE_REGISTER: {
                registerHandler_OMP(msgDto);
                break;
            }
            // 设备位置信息上报
            case Constant.M_CODE_UPLOAD_LOCATION: {
                uploadLocationHandler(msgDto);
                break;
            }
            //设备上传通话记录
            case Constant.M_CODE_UPLOAD_CALL_RECORD: {
                uploadCallRecordsHandler(msgDto);
                break;
            }
        }
        //转发消息至网页,忽略注册的报文
        if (!topic.equals(Constant.TOPIC_REGISTER_SERVER)) {
            webSocketServer.massMessage(msgDto.getDeviceId(), payload);
        }
    }

    /**
     * 注册设备
     * 老人机
     */
    private void registerHandler_OMP(MqttMsgDto obj) {
        String num = obj.getString("regnum");
        //查询数据库中是否有此设备
        Device device = deviceService.getByNum(num);
        //没有该设备,保存至数据库
        if (device == null) {
            device = new Device();
            device.setNum(num);
            device.setClientId(Constant.OLD_PHONE_DEVICE_ID_PREFIX + num);
            device.setPassword(MD5Util.MD5Encode(num));
            deviceService.save(device);
            SetBase setBase = new SetBase();
            setBase.setDeviceId(device.getClientId());
            setBaseService.save(setBase);
        }
        //mqtt服务器注册用户
        rabbitMQApi.registerMqtt(device.getClientId(), device.getPassword());
        //发送消息
        setService.returnCode_OMP(device);
    }

    /**
     * 处理上传位置
     * 老人机
     */
    private void uploadLocationHandler(MqttMsgDto msg) {
        try {
            Location location = new Location();
            location.setDeviceId(msg.getDeviceId());
            location.setBatteryNum(msg.getInt("batterynum"));
            location.setSignalNum(msg.getInt("signalnum"));
            location.setMode(msg.getInt("mode"));
            location.setLon(msg.getString("lon"));
            location.setLat(msg.getString("lat"));
            location.setSpeed(msg.getString("speed"));
            location.setNum(msg.getInt("num"));
            location.setUploadTime(msg.getDate("createtime"));
            //转换后的坐标
            String[] gcj = CoordTransformUtil.wgs84toGcj02(location.getLon(), location.getLat());
            location.setLonGcj(gcj[0]);
            location.setLatGcj(gcj[1]);
            // 逆地理位置解析
            location.setSite(GaoDeApiUtil.regeo(location.getLonGcj(), location.getLatGcj()));
            // 保存位置信息
            deviceLocationService.saveLocation(location);
        } catch (Exception e) {
            log.error("老人机上传位置信息异常", e);
        }
    }

    /**
     * 设备上传通话记录
     */
    private void uploadCallRecordsHandler(MqttMsgDto obj) {
        CallRecords callRecords = new CallRecords();
        callRecords.setDeviceId(obj.getDeviceId());
        callRecords.setCallType(obj.getInt("type"));
        callRecords.setKeyNum(obj.getString("key"));
        callRecords.setPhoneNum(obj.getString("phnoenum"));
        callRecords.setTalkTime(obj.getInt("talktime"));
        callRecords.setEndTime(obj.getString("endtime"));
        callRecordsService.save(callRecords);
    }
}
